package at.ac.uibk.gitsearch.service;

import static org.hamcrest.Matchers.anyOf;
import static org.hamcrest.Matchers.containsString;
import static org.hamcrest.Matchers.everyItem;
import static org.hamcrest.Matchers.hasItemInArray;
import static org.hamcrest.Matchers.hasProperty;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import at.ac.uibk.gitsearch.service.dto.SearchInputMetadataDTO;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.service.dto.SearchInputDTO;
import at.ac.uibk.gitsearch.service.dto.SearchResultsDTO;

/**
 * Integration tests for {@link MailService}.
 */
@SpringBootTest(classes = GitsearchApp.class)
public class SearchServiceIT {

    @Autowired
    private SearchService searchService;

    @BeforeEach
    public void setup() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void testFullTextSearch() throws Exception {
        final String WIEN = "Wien";
        final SearchInputMetadataDTO searchMetadata =
            new SearchInputMetadataDTO(null, null, null, null, null);
        SearchInputDTO searchQuery =
            new SearchInputDTO(WIEN, searchMetadata, null, null, null, 0);
        SearchResultsDTO searchResultPage =
            searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

        org.junit.Assert.assertNotNull(searchResultPage.getSearchResult());
        org.junit.Assert.assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
        org.junit.Assert.assertThat(searchResultPage.getSearchResult(), everyItem(
            anyOf(hasProperty("description", containsString(WIEN)),
                hasProperty("title", containsString(WIEN))
                // hasProperty("keywords", containsString("Wien"))
            )));

    }

    @Test
    public void testSearchByAutor() throws Exception {
        final String PODLIPNIG = "Stefan Podlipnig";
        final SearchInputMetadataDTO searchMetadata =
            new SearchInputMetadataDTO(null, null, null, null, PODLIPNIG);
        SearchInputDTO searchQuery =
            new SearchInputDTO(null, searchMetadata, null, null, null, 0);
        SearchResultsDTO searchResultPage =
            searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

        org.junit.Assert.assertNotNull(searchResultPage.getSearchResult());
        org.junit.Assert.assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
        org.junit.Assert.assertThat(searchResultPage.getSearchResult(), everyItem(
            hasProperty("creator", hasProperty("name", containsString(PODLIPNIG))))
            );

    }
    @Test
    public void testProgrammingLanguageSearch() throws Exception {
        final SearchInputMetadataDTO searchMetadata =
            new SearchInputMetadataDTO("JAVA", null, null, null, null);
        SearchInputDTO searchQuery =
            new SearchInputDTO(null, searchMetadata, null, null, null, 0);
        SearchResultsDTO searchResultPage =
            searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

        assertNotNull(searchResultPage.getSearchResult());
        assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
        assertThat(searchResultPage.getSearchResult(), everyItem(
            hasProperty("programmingLanguage",
                hasItemInArray(containsString("JAVA")))
        ));

    }

    @Test
    public void testCreatorSearch() throws IOException {
        final SearchInputMetadataDTO searchMetadata =
            new SearchInputMetadataDTO(null, null, null, null, "Podlipnig");
        SearchInputDTO searchQuery =
            new SearchInputDTO(null, searchMetadata, null, null, null, 0);
        SearchResultsDTO searchResultPage =
            searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

        org.junit.Assert.assertNotNull(searchResultPage.getSearchResult());
        org.junit.Assert.assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
        assertThat(searchResultPage.getSearchResult(), everyItem(
            hasProperty("creator", hasItemInArray(hasProperty("name", containsString("Podlipnig"))))
        ));

    }

    @Test
    public void testKeywordSearch() throws IOException {
        final SearchInputMetadataDTO searchMetadata =
            new SearchInputMetadataDTO(null, "latex", null, null, null);
        SearchInputDTO searchQuery =
            new SearchInputDTO(null, searchMetadata, null, null, null, 0);
        SearchResultsDTO searchResultPage =
            searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

        org.junit.Assert.assertNotNull(searchResultPage.getSearchResult());
        org.junit.Assert.assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
        assertThat(searchResultPage.getSearchResult(), everyItem(
            hasProperty("keyword", hasItemInArray(containsString("latex")))
        ));

    }

    @Test
    public void testLicenseSearch() throws IOException {
        final SearchInputMetadataDTO searchMetadata =
            new SearchInputMetadataDTO(null, null, null, "MIT", null);
        SearchInputDTO searchQuery =
            new SearchInputDTO(null, searchMetadata, null, null, null, 0);
        SearchResultsDTO searchResultPage =
            searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

        org.junit.Assert.assertNotNull(searchResultPage.getSearchResult());
        org.junit.Assert.assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
        assertThat(searchResultPage.getSearchResult(), everyItem(
            hasProperty("license", containsString("MIT"))
        ));

    }
}
